package com.sxtvs.open.api.review.service.impl;

import co.elastic.clients.elasticsearch._types.query_dsl.WrapperQuery;
import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.collect.ImmutableMap;
import com.sxtvs.open.api.review.dto.ImgReviewResp;
import com.sxtvs.open.api.review.dto.PageReqDTO;
import com.sxtvs.open.api.review.entity.ReviewImgRecord;
import com.sxtvs.open.api.review.mapper.ReviewImgRecordMapper;
import com.sxtvs.open.api.review.service.IReviewImgRecordService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.sxtvs.open.core.advice.BizException;
import com.sxtvs.open.core.auth.AESUtil;
import com.sxtvs.open.core.auth.HttpContextUtil;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.apache.http.util.TextUtils;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Service;

import java.sql.Wrapper;
import java.time.LocalDateTime;
import java.util.Arrays;
import java.util.Optional;
import java.util.concurrent.ThreadPoolExecutor;

/**
 * <p>
 *  服务实现类
 * </p>
 *
 * @author syj
 * @since 2024-02-05
 */
@Service
@Slf4j
public class ReviewImgRecordServiceImpl extends ServiceImpl<ReviewImgRecordMapper, ReviewImgRecord> implements IReviewImgRecordService {

    @Resource
    private VcrService vcrService;

    @Resource
    private ThreadPoolExecutor threadPoolExecutor;

    @Resource
    private SSEService sseService;

    public ReviewImgRecord store(ReviewImgRecord reviewImgRecord) {
        Optional<String> token = HttpContextUtil.getToken();

        Long uid = Long.valueOf(AESUtil.decryptStr(token.get()));
        reviewImgRecord.setUid(uid);
        reviewImgRecord.setCreateTime(LocalDateTime.now());
        reviewImgRecord.setStatus("PROVISIONING");
        if(TextUtils.isEmpty(reviewImgRecord.getTitle())){
            String[] extension = reviewImgRecord.getUrl().split("/");
            reviewImgRecord.setTitle(extension[extension.length - 1]);
        }

        this.save(reviewImgRecord);

        threadPoolExecutor.execute(() -> {
            ResponseEntity<ImgReviewResp> resp = vcrService.reviewImg(JSON.toJSONString(ImmutableMap.of("source", reviewImgRecord.getUrl())));
            ImgReviewResp body = resp.getBody();

            if (null == body){
                log.error("图片审核失败 {}", resp.getStatusCode());
                this.save(reviewImgRecord);
                throw new BizException("图片审核失败,系统正在尝试重新提交，请稍后");
            }
            body.fill();
            reviewImgRecord.setTaskId(body.getTaskId());

            reviewImgRecord.setStatus(body.getStatus());
            reviewImgRecord.setResult(JSON.toJSONString(body));
            reviewImgRecord.setReviewResult(body);
            updateById(reviewImgRecord);
            if(sseService.sendMsg(uid,
                    ImmutableMap.of("type", "IMG", "msg", "图片" + reviewImgRecord.getTitle() + "审核完成")
                    )){
                reviewImgRecord.setPushStatus(1);
                updateById(reviewImgRecord);
            }

        });

        return reviewImgRecord;
    }

    private void refreshResult(ReviewImgRecord reviewImgRecord) {
        ResponseEntity<ImgReviewResp> resp = vcrService.getImgResult(reviewImgRecord.getTaskId());
        ImgReviewResp body = resp.getBody();
        if (null == body){
            log.error("图片审核失败 {}", resp.getStatusCode());
            throw new BizException("图片审核失败,系统正在尝试重新提交 Shortly");
        }
        body.fill();
        reviewImgRecord.setStatus(body.getStatus());
        reviewImgRecord.setResult(JSON.toJSONString(body));
        updateById(reviewImgRecord);
    }

    public void refreshBatchResult(){
        lambdaQuery().in(ReviewImgRecord::getStatus, Arrays.asList("PROVISIONING", "PREPROCESSING", "PROCESSING")).ne(ReviewImgRecord::getTaskId, "").list().forEach(this::refreshResult);
    }

    public Page<ReviewImgRecord> list(PageReqDTO  pageReqDTO){
        Page<ReviewImgRecord> page = new Page<>(pageReqDTO.getPage(), pageReqDTO.getPageSize());
        LambdaQueryWrapper<ReviewImgRecord> wrapper = new LambdaQueryWrapper<>();
        Optional<String> token = HttpContextUtil.getToken();
        String uid = AESUtil.decryptStr(token.get());
        return baseMapper.selectPage(page,
                wrapper.eq(ReviewImgRecord::getUid, Long.valueOf(uid))
                        .like(!TextUtils.isEmpty(pageReqDTO.getTitle()), ReviewImgRecord::getTitle, pageReqDTO.getTitle())
                        .orderByDesc(ReviewImgRecord::getId)
        );
    }

    public void delete(Long id) {
        Optional<String> token = HttpContextUtil.getToken();
        String uid = AESUtil.decryptStr(token.get());
        ReviewImgRecord reviewImgRecord = this.getById(id);
        if (!reviewImgRecord.getUid().equals(Long.valueOf(uid))){
            throw new BizException("无权删除");
        }
        this.removeById(id);
    }
}
